<?php
class ModelExtensionShippingInxpressShippingRates extends Model {
	function getQuote($address) {
		$this->load->language('extension/shipping/inxpress_shipping_rates');

		if ($this->config->get('shipping_inxpress_shipping_rates_zone_all')) {
			$status = true;
		} else {
			$status = $this->check_geo_zones($address);
		}

		if (!$status) {
			return array();
		};

	 	$method_data = array();
		$items = array();

		$products = $this->cart->getProducts();
		foreach ( $products as $item ) {
			array_push(
				$items,
				array(
					'id'          => $item['product_id'],
					'name'        => $item['name'],
					'weight'      => $this->calculate_weight_in_grams($item['weight'], $item['weight_class_id']),
					'height'      => $this->calculate_dim_in_mm($item['height'], $item['length_class_id']),
					'width'       => $this->calculate_dim_in_mm($item['width'], $item['length_class_id']),
					'depth'       => $this->calculate_dim_in_mm($item['length'], $item['length_class_id']),
					'price'       => $item['price'],
					'weight_unit' => $item['weight_class_id'],
					'dim_unit'    => $item['length_class_id'],
					'quantity'    => $item['quantity']
				)
			);
		}

		$services = array();
		if ($this->config->get('shipping_inxpress_shipping_rates_dhl')) {
			array_push($services, array(
				'carrier' => 'DHL',
				'service' => 'DHL Express'
			));
		}
		if ($this->config->get('shipping_inxpress_shipping_rates_ups')) {
			array_push($services, array(
				'carrier' => 'UPS',
				'service' => 'UPS Express'
			));
		}
		if ($this->config->get('shipping_inxpress_shipping_rates_purolator')) {
			array_push($services, array(
				'carrier' => 'Purolator',
				'service' => 'Purolator Express'
			));
		}
		if ($this->config->get('shipping_inxpress_shipping_rates_canpar')) {
			array_push($services, array(
				'carrier' => 'Canpar',
				'service' => 'Canpar Express'
			));
		}

		$origin = array(
			'city'        => $this->config->get('shipping_inxpress_shipping_rates_city'),
			'province'    => $this->config->get('shipping_inxpress_shipping_rates_state'),
			'country'     => $this->config->get('shipping_inxpress_shipping_rates_country'),
			'postal_code' => $this->config->get('shipping_inxpress_shipping_rates_postcode')
		);

		$destination = array(
			'name'        => '',
			'address1'    => $address['address_1'],
			'address2'    => $address['address_2'],
			'city'        => $address['city'],
			'province'    => $address['zone'],
			'country'     => $address['iso_code_2'],
			'postal_code' => $address['postcode']
		);

		$rates = $this->fetch_rates( $this->config->get('shipping_inxpress_shipping_rates_inxpress_account'), $services, $origin, $destination, $items );

		if ( !$rates ) {
			return array();
		}

		$quote_data = array();

		foreach($rates as $rate) {
			$quote_data[$rate['service_code']] = array(
				'code'         => 'inxpress_shipping_rates.' . $rate['service_code'],
				'title'        => $rate['display_text'],
				'cost'         => $rate['total_price']/100,
				'tax_class_id' => $this->config->get('shipping_inxpress_shipping_rates_tax_class_id'),
				'text'         => $this->currency->format($this->tax->calculate($rate['total_price']/100, $this->config->get('shipping_inxpress_shipping_rates_tax_class_id'), $this->config->get('config_tax')), $this->session->data['currency'])
			);
		}

		$method_data = array(
			'code'       => 'inxpress_shipping_rates',
			'title'      => $this->language->get('text_title'),
			'quote'      => $quote_data,
			'sort_order' => $this->config->get('shipping_inxpress_shipping_rates_sort_order'),
			'error'      => false
		);

		return $method_data;
	}

	public function fetch_rates($account, $services, $origin, $destination, $products) {
		$request = json_encode(
			array(
				'account'     => $account,
				'gateway'     => $origin['country'],
				'services'    => $services,
				'origin'      => $origin,
				'destination' => $destination,
				'items'       => $products,
			)
		);

		$store_id = $this->config->get('inxpress_shipping_store_id');
		$url = 'https://api.inxpressapps.com/carrier/v1/stores/' . $store_id . '/rates';

		$curl = curl_init($url);

		curl_setopt($curl, CURLOPT_HEADER, 0);
		curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_TIMEOUT, 60);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $request);
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));

		$result = curl_exec($curl);

		curl_close($curl);

		$response = json_decode( $result, true );

		if ( isset( $response['rates'] ) ) {
			return $response['rates'];
		} else {
			return false;
		}
	}

	public function calculate_weight_in_grams( $weight_in_uom, $unit_id ) {
		switch ( $unit_id ) {
			case '6':
				$weight = $weight_in_uom * 28.3495;
				break;
			case '5':
				$weight = $weight_in_uom * 453.5920;
				break;
			case '1':
				$weight = $weight_in_uom * 1000;
				break;
			default:
				$weight = $weight_in_uom;
		}

		return $weight;
	}

	public function calculate_dim_in_mm( $dim, $dim_unit ) {
		switch ( $dim_unit ) {
			case '3':
				$con_dim = $dim * 25.4;
				break;
			case '1':
				$con_dim = $dim * 10;
				break;
			default:
				$con_dim = $dim;
		}

		return round( $con_dim, 1 );
	}

	function check_geo_zones($address) {
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "geo_zone ORDER BY name");

		$status = false;
		foreach ($query->rows as $result) {
			if ($this->config->get('shipping_inxpress_shipping_rates_zone_' . $result['geo_zone_id'])) {
				$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "zone_to_geo_zone WHERE geo_zone_id = '" . (int)$result['geo_zone_id'] . "' AND country_id = '" . (int)$address['country_id'] . "' AND (zone_id = '" . (int)$address['zone_id'] . "' OR zone_id = '0')");

				if ($query->num_rows) {
					$status = true;
				}
			}
		}

		return $status;
	}
}
